#include <oxstd.h>
#import <packages/Garch6/garch>
#include <oxdraw.h>
#include <oxfloat.h>
#import <packages/arfima/arfima>

without(const startPoint,const endPoint, const ForcLength);//The function that produces forecasts for x days.


main()
{
	decl i=0;
	decl length=5;//Number of days to simulate
	while(i*length+length<751)//Runs through the out of sample observations
	{
		without(1+i*length,3892-750+i*length,length);
		i++;
	}
}



without(const startPoint,const endPoint, const ForcLength)
{

	decl start=startPoint;
	decl end=endPoint;
	println(start,", ",end);
	decl length=end-start+1;

	//declares the main EGARCH model
	decl egarch = new Garch();

	egarch.Load("\\\\kain.stud.iot.ntnu.no\\homedir$\\torsteri\\MyDocs\\Dropbox\\MASTER\\Data\\New estimations\\All data.xls");
	egarch.Deterministic(-1);

	decl par,errors,Sigma2,returns;
	decl check=3;


	//This while loop makes sure the model converges, if it does not, it moves the start of the dataset one day and tries again	
	while(check>2)
	{
		decl garch = new Garch();
		garch.Load("\\\\kain.stud.iot.ntnu.no\\homedir$\\torsteri\\MyDocs\\Dropbox\\MASTER\\Data\\New estimations\\All data.xls");
		garch.Select(Y_VAR, {"NG ret", 0, 0});
		garch.CSTS(1,1);
		garch.DISTRI(3);
		garch.ARMA_ORDERS(1,0);
		garch.ARFIMA(0);
		garch.GARCH_ORDERS(1,1);
		garch.MODEL(EGARCH);
		garch.MLE(2);
		garch.ITER(0);
		garch.SetSelSampleByIndex(start,end);
		garch.Initialization(<>);
		garch.DoEstimation(<>);
		garch.Output();
		check=garch.m_iResult;
		if(check>2)
		{
			start=start+1;
		}
		par=garch.GetValue("m_vPar");
		errors=garch.GetValue("m_vE");
		Sigma2=log(garch.GetValue("m_vSigma2"));
		returns=garch.GetGroup(Y_VAR);


		delete garch;
	}

	//Saves each of the coefficients as its own name to make programming easier
	decl CstM=par[0];
	decl AR=par[1];
	decl CstV=par[2];
	decl Alpha =par[3];
	decl Beta =par[4];
	decl Theta1=par[5];
	decl Theta2=par[6];
	decl Asymm=par[7];
	decl Tail =par[8];

	
	decl r1,r2,e1,h1,h,sd,lnh;
	decl detImpact,ldetImpact;

	

	decl p=1,q=1;
	decl i,j;
	
	//Declares the number of simulations, and creates vectors with room for simulated returns and volatility
	decl T=rows(errors)-1;
	decl simulations=11000;
	decl numOfForc = ForcLength;  
	decl sigma_sim = Sigma2|zeros(numOfForc,1);
	decl sim_returns=returns|zeros(numOfForc,1);

	//Declares the error variables, z variables needed for the simulation. As well as the matrix for storing simulated determinants
	decl v,e,eOil,eSP,eStock,eProd, eDD;
	decl z0,zROil,zRSP,zProd,zStock,zDD,zVOil,zVSP;
	decl detValue=zeros(numOfForc,7),SigmaOil=zeros(numOfForc,1);
	decl SigmaSP=zeros(numOfForc,1);

	

	//Saving the mean and st.dev for the determinants to be simulated
	decl descDD=moments(egarch.GetVar("DD diff")[start:end],2);
	decl descROil=moments(egarch.GetVar("Oil ret")[start:end],2);
	decl descRSP=moments(egarch.GetVar("SP ret")[start:end],2);


	//Declares the g(z) function for use in EGARCH
	decl g,lg;

	//Declares the matricies for storing simulates returns and volatility
	decl saveSigma,saveReturn=zeros(numOfForc,simulations),tempSigma=zeros(numOfForc,simulations),tempReturn=zeros(numOfForc,simulations);


	//Calculates the value of E(abs(z)), which is used in the g(z) function in EGARCH
	decl as = exp(Asymm);  
	decl b = 2/((1/as)+as); 
	decl t1 = 2*b*sqrt(Tail-2)*(as^2)*gammafact((1+Tail)/2); 
	decl t2 = sqrt(M_PI)*(Tail - 1)*gammafact(Tail/2); 
	decl EabsZ = t1 / t2;


	//Declares the W variable, for the model without determinants, this is simply omega
	decl W=0,lW=0;
	
	//Runs through i days and j simulations of those i days
	for(j=0;j<simulations;j++)
	{
		for (i=0;i<numOfForc;i++)  
		{  
			z0=egarch.rndmskewedt(1,1,Tail,exp(Asymm));
			
			e=z0*sqrt(exp(sigma_sim[T+i][0]));
			sim_returns[T+i]=CstM+AR*sim_returns[T+i-1]+e;
			
			g=Theta1*z0+Theta2*(fabs(z0)-EabsZ);
			if(i==0)
				lg=g;

			sigma_sim[T+i+1][0] = CstV+g+Alpha*lg+Beta*(sigma_sim[T+i]-CstV);
			lg=g;
		}
		

		//For each simulation, simulated values of returns and volatility
		for(i=0;i<numOfForc;i++)
		{
			tempSigma[i][j]=exp(sigma_sim[T+i][0]);
			tempReturn[i][j]=sim_returns[T+i];
		
		}			
	}

	//Convert start and numOfForc to strings such that they can be used in the filenames stored
	decl s=sprint(start);
	decl F=sprint(numOfForc);


	 //Saves the matix of simulated volatility and returns in the excel files specified
	savemat("sigma"~s~"with"~F~"forecasts.xls",tempSigma');
	savemat("returns"~s~"with"~F~"forecasts.xls",tempReturn');
	println("DONE");

	delete egarch;


}